##############################################################################
#
#
#                     Structured Text Document Product
#              for the Z Object Publishing Environment (Zope)
#
#
# (c) Copyright 2001 by Philipp "philiKON" von Weitershausen
#                       philikon@gmx.net
#                       http://www.philikon.de
#
#
# Released unter the terms of the GPL v2
# See COPYING.txt for license text
#
#
# Please see README.txt for further information
#
#
##############################################################################




from string import atoi
import Acquisition
import Globals
from string import atoi
from AccessControl import Role, ClassSecurityInfo
from OFS import SimpleItem
from OFS.PropertyManager import PropertyManager
from webdav.Lockable import ResourceLockedError
from webdav.WriteLockInterface import WriteLockInterface
from DateTime import DateTime




class STXDocument ( Acquisition.Implicit,
                    Globals.Persistent,
                    Role.RoleManager,
                    PropertyManager,
                    SimpleItem.Item    ):
  """ Structured Text Document class """


  __implements__ = ( WriteLockInterface, )
  meta_type = "Structured Text Document"
  security = ClassSecurityInfo()


  manage_options = (
    ( {'label' : 'Edit', 'action' : 'manage_main' },
      {'label' : 'View', 'action' : ''  } )
    + PropertyManager.manage_options
    + SimpleItem.Item.manage_options
    + Role.RoleManager.manage_options
  )


  __ac_permissions__=(
    ( 'View management screens', ( 'document_src', 'PrincipiaSearchSource' ) ),
    ( 'Change Structured Text Documents', ( 'manage', 'manage_main', 'manage_edit', 'manage_upload', 'PUT' ) ),
    ( 'View', ( 'index_html', '__call__', 'get_size', ) ),
    ( 'FTP access', ( 'manage_FTPstat', 'manage_FTPget', 'manage_FTPlist' ) )
  )


  manage_main = Globals.DTMLFile( "dtml/documentEdit", globals() )
  index_html = Globals.DTMLFile( "dtml/index_html", globals() )


  def __init__( self, id, title, stx_src ):
    self.id = id
    self.title = title
    self.munge( stx_src )



  ## Different methods for returning/editing the source code ##


  def munge( self, source_string=None ):
    """ Save the document source"""
    if source_string is not None:
      self.raw = source_string


  def read( self ):
    return self.raw


  def get_size( self ):
    return len( self.raw )


  def PrincipiaSearchSource( self ):
    """ Support for searching - the document's contents are searched. """
    return self.read()


  def document_src( self, REQUEST=None, RESPONSE=None ):
    """ Return unprocessed document source. """
    if RESPONSE is not None:
      RESPONSE.setHeader( 'Content-Type', 'text/plain' )
    return self.read()


  def __str__( self ):
    return self.read()



  ## Zope Management Interface (ZMI) methods ##


  def manage_edit( self, data, title, submit='Change', stx_pref_cols='50', stx_pref_rows='20', REQUEST=None ):
    """ Replaces a document's contents with data, the title with title"""

    if self._size_changes.has_key( submit ):
      return self._er( data, title, submit, stx_pref_cols, stx_pref_rows, REQUEST)

    if self.wl_isLocked():
       raise ResourceLockedError, ( 'This document has been locked via WebDAV.' )

    self.title = str( title )
    if type( data ) is not type( '' ):
      data = data.read()

    self.munge( data )

    if REQUEST is not None:
      message = "Content changed."
      return self.manage_main( self, REQUEST, manage_tabs_message=message )


  _size_changes={
        'Bigger': (5,5),   
        'Smaller': (-5,-5),
        'Narrower': (0,-5),
        'Wider': (0,5),   
        'Taller': (5,0),  
        'Shorter': (-5,0),
  }

  def _er( self, data, title, submit, stx_pref_cols, stx_pref_rows, REQUEST):
    """ Change size of input field """
    dr, dc = self._size_changes[ submit ]
        
    rows = max( 1, atoi( stx_pref_rows ) + dr )  
    cols = max( 40, atoi( stx_pref_cols) + dc ) 
    e = ( DateTime('GMT') + 365 ).rfc822()
    resp = REQUEST['RESPONSE']
    resp.setCookie( 'stx_pref_rows', str(rows), path='/', expires=e )
    resp.setCookie( 'stx_pref_cols', str(cols), path='/', expires=e )

    return self.manage_main( self, REQUEST, title=title, __str__=data, stx_pref_cols=cols, stx_pref_rows=rows )


  def manage_upload(self,file='', REQUEST=None):
    """Replace the contents of the document with the text in file."""

    if self.wl_isLocked():
      raise ResourceLockedError, ( 'This document has been locked via WebDAV.' )

    if type( file ) is not type(''):
      file = file.read()

    self.munge(file)

    if REQUEST is not None:
      message = "Content uploaded."
      return self.manage_main(self, REQUEST, manage_tabs_message=message )



  ## Protocol handlers ##


  def PUT( self, REQUEST, RESPONSE ):
    """ Handle HTTP PUT requests. """
    self.dav__init( REQUEST, RESPONSE )
    self.dav__simpleifhandler( REQUEST, RESPONSE, refresh=1 )
    body = REQUEST.get( 'BODY', '' )
    self.munge( body )
    RESPONSE.setStatus( 204 )
    return RESPONSE


  def manage_FTPget( self ):
    """ Get source for FTP download """
    return self.read()



# initialize the product
Globals.InitializeClass( STXDocument )




### object constructors ###

documentAdd = Globals.DTMLFile( "dtml/documentAdd", globals() )

def addSTXDocument( self, id, title="", file="", REQUEST=None ):
  """ Create a STXDocument instance to an ObjectManager """

  if type( file ) is not type( "" ):
    file = file.read()

  obj_instance = STXDocument( id, title, file )
  self.Destination()._setObject( id, obj_instance )

  if REQUEST is None:
    return

  try:
    url = self.DestinationURL()
  except:
    url = REQUEST['URL1']
  if REQUEST['submit'] == " Add and Edit ":
    url = "%s/%s" % ( url, id )
  REQUEST.RESPONSE.redirect( url + '/manage_main' )

